
import {history} from "../../../../../node_modules/prosemirror-history";
import {keymap} from "../../../../../node_modules/prosemirror-keymap";
import {dropCursor} from "../../../../../node_modules/prosemirror-dropcursor";
import {gapCursor} from "../../../../../node_modules/prosemirror-gapcursor";
import {buildKeymap, buildInputRules} from "../../../../../node_modules/prosemirror-example-setup";
import {baseKeymap, chainCommands} from "../../../../../node_modules/prosemirror-commands";
import {nodes, marks} from '../../../../../node_modules/prosemirror-schema-basic';
import {Schema} from '../../../../../node_modules/prosemirror-model';
import {addListNodes} from '../../../../../node_modules/prosemirror-schema-list';
import {defaultMarkdownSerializer} from '../../../../../node_modules/prosemirror-markdown';
import {menuBar} from '../../../../../node_modules/prosemirror-menu';
import {Plugin} from '../../../../../node_modules/prosemirror-state';

import {buildMenuItems} from "./menu";
import {pasteImage} from "./plugins";


export {buildMenuItems};


// PLUGINS
// options::- The following options are recognized:
//
//   schema:: Schema
//   The schema to generate key bindings and menu items for.
//
//   mapKeys:: ?Object
//   Can be used to [adjust](#example-setup.buildKeymap) the key bindings created.
//
//   menuBar:: ?bool
//   Set to false to disable the menu bar.
//
//   history:: ?bool
//   Set to false to disable the history plugin.

export function basicSetupPlugin(options) {
  let plugins = [
    // basic input rules
    buildInputRules(options.schema),
    dropCursor(),
    gapCursor()
  ];

  // custom keymaps
  if (options.customKeymap) {
    Object.keys(options.customKeymap).forEach(function(key) {
      if (baseKeymap[key]) {
        options.customKeymap[key] = chainCommands(baseKeymap[key], options.customKeymap[key]);
      }
    });
    plugins.push(keymap(options.customKeymap));
  }

  // basic keymaps
  plugins.push(
    keymap(buildKeymap(options.schema, options.mapKeys)),
    keymap(baseKeymap)
  );

  // paste image
  if (options.pasteImage !== false) plugins.push(pasteImage());

  // history (undo)
  if (options.history !== false) plugins.push(history());

  // menu bar
  if (options.menuBar !== false) {
    plugins.push(menuBar({
      floating: false,
      content: buildMenuItems(options.schema).fullMenu
    }));
  }

  if (options.class) {
    plugins.push(new Plugin({
      props: {
        attributes: {class: options.class}
      }
    }));
  }

  return plugins;
}

// SCHEMA
let customNodes = nodes;
customNodes.uploading_image = {
  inline: true,
  attrs: {
    src: {},
    alt: {default: null},
    title: {default: null}
  },
  group: "inline",
  draggable: true,
  toDOM(node) {
    return [
      "span", {class: 'ProseMirror-uploading-image'}, ["img", node.attrs],
      ["i", {class:'fa fa-spinner fa-spin'}]
    ];
  }
};

let schema = new Schema({nodes: customNodes, marks: marks});
export let basicSchemaSpec = {
  nodes: addListNodes(schema.spec.nodes, 'paragraph block*', 'block'),
  marks: schema.spec.marks
};

// TOKENS FOR MARKDOWN PARSER
export let basicMarkdownParserTokens = {
  blockquote: {block: 'blockquote'},
  paragraph: {block: 'paragraph'},
  list_item: {block: 'list_item'},
  bullet_list: {block: 'bullet_list'},
  ordered_list: {block: 'ordered_list', getAttrs: tok => ({order: +tok.attrGet('order') || 1})},
  heading: {block: 'heading', getAttrs: tok => ({level: +tok.tag.slice(1)})},
  code_block: {block: 'code_block'},
  fence: {block: 'code_block'},
  hr: {node: 'horizontal_rule'},
  image: {node: 'image', getAttrs: tok => ({
    src: tok.attrGet('src'),
    title: tok.attrGet('title') || null,
    alt: tok.children[0] && tok.children[0].content || null
  })},
  hardbreak: {node: 'hard_break'},
  em: {mark: 'em'},
  strong: {mark: 'strong'},
  link: {mark: 'link', getAttrs: tok => ({
    href: tok.attrGet('href'),
    title: tok.attrGet('title') || null
  })},
  code_inline: {mark: 'code'}
};

// MARKDOWN SERIALIZER INSTANCE
var basicMarkdownSerializer = Object.assign(Object.create(defaultMarkdownSerializer), defaultMarkdownSerializer);

// Patch hard_break to only be \n instead of a "\" then a \n
basicMarkdownSerializer.nodes.hard_break = function(state, node, parent, index) {
  for (let i = index + 1; i < parent.childCount; i++) {
    if (parent.child(i).type != node.type) {
      state.write("\n");
      return;
    }
  }
};

basicMarkdownSerializer.nodes.uploading_image = function(state, node) {
  state.write("![" + state.esc(node.attrs.alt || "") + "](" + state.esc(node.attrs.src) +
              (node.attrs.title ? " " + state.quote(node.attrs.title) : "") + ")");
};

export {basicMarkdownSerializer};
