
import {wrapItem, blockTypeItem, MenuItem} from "../../../../../node_modules/prosemirror-menu";
import {NodeSelection} from "../../../../../node_modules/prosemirror-state";
import {toggleMark} from "../../../../../node_modules/prosemirror-commands";
import {wrapInList} from "../../../../../node_modules/prosemirror-schema-list";
import crel from '../../../../../node_modules/crel';

import {MenuDropItem, renderDropdownItems} from "../ui/menu";
import {TextField, openPrompt} from "../ui/form-prompt";

function wrapListItem(nodeType, options) {
  return cmdItem(wrapInList(nodeType, options.attrs), options);
}

function canInsert(state, nodeType, attrs) {
  let $from = state.selection.$from;
  for (let d = $from.depth; d >= 0; d--) {
    let index = $from.index(d);
    if ($from.node(d).canReplaceWith(index, index, nodeType, attrs)) return true;
  }
  return false;
}

function insertImageItem(nodeType) {
  return new MenuItem({
    title: "Insert image",
    icon: {dom:crel('i', {class: 'fa fa-image'})},
    enable(state) { return canInsert(state, nodeType); },
    run(state, _, view) {
      let {from, to} = state.selection, attrs = null;
      if (state.selection instanceof NodeSelection && state.selection.node.type == nodeType)
        attrs = state.selection.node.attrs;
      openPrompt(view, {
        title: "Insert image",
        fields: {
          src: new TextField({label: "Location", required: true, value: attrs && attrs.src}),
          title: new TextField({label: "Title", value: attrs && attrs.title}),
          alt: new TextField({label: "Description",
                              value: attrs ? attrs.title : state.doc.textBetween(from, to, " ")})
        },
        callback(attrs) {
          view.dispatch(view.state.tr.replaceSelectionWith(nodeType.createAndFill(attrs)));
          view.focus();
        }
      });
    }
  });
}

function cmdItem(cmd, options) {
  let passedOptions = {
    label: options.title,
    run: cmd
  };
  for (let prop in options) passedOptions[prop] = options[prop];
  if ((!options.enable || options.enable === true) && !options.select)
    passedOptions[options.enable ? "enable" : "select"] = state => cmd(state);
  return new MenuItem(passedOptions);
}

function markActive(state, type) {
  let {from, $from, to, empty} = state.selection;
  if (empty) return type.isInSet(state.storedMarks || $from.marks());
  else return state.doc.rangeHasMark(from, to, type);
}

function markItem(markType, options) {
  let passedOptions = {
    active(state) { return markActive(state, markType); },
    enable: true
  };
  for (let prop in options) passedOptions[prop] = options[prop];
  return cmdItem(toggleMark(markType), passedOptions);
}

function linkItem(markType) {
  return new MenuItem({
    title: "Add or remove link",
    icon: {dom:crel('i', {class: 'fa fa-link'})},
    active(state) { return markActive(state, markType); },
    enable(state) { return !state.selection.empty; },
    run(state, dispatch, view) {
      if (markActive(state, markType)) {
        toggleMark(markType)(state, dispatch);
        return true;
      }
      openPrompt(view, {
        title: "Create a link",
        fields: {
          href: new TextField({
            label: "Link target",
            required: true,
            clean: (val) => {
              if (!/^https?:\/\//i.test(val))
                val = 'http://' + val;
              return val;
            }
          }),
          title: new TextField({label: "Title"})
        },
        callback(attrs) {
          toggleMark(markType, attrs)(view.state, view.dispatch);
          view.focus();
        }
      });
    }
  });
}

export function buildMenuItems(schema) {
  let r = {};
  let type;

  if (type = schema.marks.strong) {
    r.toggleStrong = markItem(type, {
      title: "Bold",
      icon: {dom:crel('i', {class: 'fa fa-bold'})}
    });
  }
  if (type = schema.marks.em) {
    r.toggleEm = markItem(type, {
      title: "Italic",
      icon: {dom:crel('i', {class: 'fa fa-italic'})}
    });
  }

  if (type = schema.marks.link) {
    r.toggleLink = linkItem(type);
  }

  if (type = schema.nodes.image) {
    r.insertImage = insertImageItem(type);
  }

  if (type = schema.nodes.bullet_list) {
    r.wrapBulletList = wrapListItem(type, {
      title: "Bullet list",
      icon: {dom:crel('i', {class: 'fa fa-list-ul'})},
      enable: true
    });
  }
  if (type = schema.nodes.ordered_list) {
    r.wrapOrderedList = wrapListItem(type, {
      title: "Ordered list",
      icon: {dom:crel('i', {class: 'fa fa-list-ol'})},
      enable: true
    });
  }

  if (type = schema.nodes.blockquote) {
    r.wrapBlockQuote = wrapItem(type, {
      title: "Block quote",
      icon: {dom:crel('i', {class: 'fa fa-quote-left'})}
    });
  }
  if (type = schema.nodes.code_block) {
    r.makeCodeBlock = blockTypeItem(type, {
      label: "Code block"
    });
  }
  if (type = schema.nodes.paragraph) {
    r.makeParagraph = blockTypeItem(type, {
      label: "Normal"
    });
  }
  if (type = schema.nodes.heading) {
    for (let i = 1; i <= 10; i++) {
      r["makeHead" + i] = blockTypeItem(type, {
        label: "Header " + i,
        attrs: {level: i}
      });
    }
  }

  function getActiveBlockLabels(view) {
    let labels = blockMenuItems.filter(item => item.spec && item.spec.active && item.spec.active(view.state))
                               .map(item => item.spec && item.spec.label);
    let menuDropItemLabel = labels.length ?
      labels.reduce((previous, current) => previous + ', ' + current) : 'Normal';

    return crel('span', null, view.props.translate(menuDropItemLabel));
  }

  let cut = arr => arr.filter(x => x);
  let blockMenuItems = cut([r.makeParagraph, r.makeHead1, r.makeHead2, r.makeHead3, r.makeCodeBlock]);
  if (r.makeHead1) {
    r.headingMenu = new MenuDropItem({
      title: "Styles",
      content: view => renderDropdownItems(blockMenuItems, view),
      render: getActiveBlockLabels,
      caret: true
    });
  }
  r.baseMenu = [r.headingMenu].concat(cut([r.toggleStrong, r.toggleEm]));
  r.insertMenu = cut([r.wrapBlockQuote, r.toggleLink, r.insertImage]);
  r.listMenu = cut([r.wrapBulletList, r.wrapOrderedList]);

  r.fullMenu = [r.baseMenu, r.insertMenu, r.listMenu];

  return r;
}

// function markActive(pm, type) {
//   let {from, to, empty} = pm.selection;
//   if (empty) return type.isInSet(pm.activeMarks());
//   else return pm.doc.rangeHasMark(from, to, type);
// }

// // WIP !!! Don't use this.
// function menuLinkItem(markType) {
//   return new MenuDropItem({
//     title: "Add or remove link",
//     icon: {dom:elt('i', {class: 'fa fa-link'})},
//     active(pm) { return markActive(pm, markType); },
//     enable(pm) { return toggleMark(markType, null)(pm, false); },
//     run(pm) {
//       if (markActive(pm, markType)) {
//         toggleMark(markType, null)(pm);
//         return true;
//       }
//     },
//     content: function(pm) {
//       let form = new MenuForm(pm, "Create a link", {
//         href: new TextField({
//           label: "Link target",
//           required: true,
//           clean: (val) => {
//             if (!/^https?:\/\//i.test(val))
//               val = 'http://' + val;
//             return val;
//           }
//         }),
//         title: new TextField({label: "Title"})
//       });

//       return form.render(pm, params => toggleMark(markType, params)(pm));
//     },
//     afterOpened: function(pm, drop) {
//       let input = drop.drop.querySelector('form').elements[0];
//       if (input) input.focus();
//     }
//   });
// }
// exports.menuLinkItem = menuLinkItem;
