
import elt from '../../../../../node_modules/crel';
import {keymap} from "../../../../../node_modules/prosemirror-keymap";
import {TooltipDrop} from '../ui/tooltip-drop';

export class MentionTooltipDrop extends TooltipDrop {
  constructor(view, spec) {
    super(view);

    let self = this;

    // spec contains : word, from, to, mentions
    this.spec = spec;
    this.items = this.createItems(spec.mentions);
    this.coords = spec.coords;
    this.selectedMentionIdx = -1;

    this.keymapPlugin = keymap({
      "ArrowUp": function(state, dispatch, view) {
        self.selectMention('up');
        return true;
      },
      "ArrowDown": function(state, dispatch, view) {
        self.selectMention('down');
        return true;
      },
      "Enter": function(state, dispatch, view) {
        if (self.selectedMentionIdx > -1) {
          self.applyCompletion(self.spec.mentions[self.selectedMentionIdx], state, dispatch);
          return true;
        }
        return false;
      },
      "Tab": function(state, dispatch, view) {
        if (self.selectedMentionIdx > -1) {
          self.applyCompletion(self.spec.mentions[self.selectedMentionIdx], state, dispatch);
          return true;
        }
        return false;
      }
    });
  }

  open() {
    super.open(elt('div', {}, this.items),
      {left: this.coords.left, top: this.coords.bottom});
    this.selectMention();

    // add drop keymap Plugin from editor's state
    let newState = this.view.state.reconfigure({
      plugins: [this.keymapPlugin].concat(this.view.state.plugins)
    });

    if (this.view.props.isIE) {
      // DO NOT USE view.updateState or focus on IE 11, seems to trigger a bug when this is called with Backspace key
      this.view.state = newState;
    } else {
      this.view.updateState(newState);
      if (!this.view.hasFocus()) this.view.focus();
    }
  }

  close() {
    let self = this;
    super.close();

    // remove drop keymap Plugin from editor's state
    let newState = this.view.state.reconfigure({
      plugins: this.view.state.plugins.filter(function(plugin) {
        return plugin.key !== self.keymapPlugin.key;
      })
    });

    if (this.view.props.isIE) {
      // DO NOT USE view.updateState or focus on IE 11, seems to trigger a bug when this is called with Backspace key
      this.view.state = newState;
    } else {
      this.view.updateState(newState);
      if (!this.view.hasFocus()) this.view.focus();
    }


  }

  destroy() {
    super.destroy();
    this.spec = null;
    this.view = null;
    this.coords = null;
    this.keymapPlugin = null;
    this.selectedMentionIdx = -1;
  }

  createItems(mentions) {
    let self = this;
    return mentions.map(function (mention) {
      let item;
      let name = elt('span', {}, mention.display_name);
      let markup = elt('span', {class: 'mention-markup'}, '@' + mention.username);
      let nameContainer = elt('div', {class: 'mention-name'}, name, markup);
      if (mention.avatar) {
        let icon = elt('img', { src: mention.avatar, class: 'icon', title: mention.username });
        item = elt('div', {class: 'mention-item'}, icon, nameContainer);
      } else {
        item = elt('div', {class: 'mention-item' }, nameContainer);
      }
      item.addEventListener('mousedown', function(e) {
        e.preventDefault();
        e.stopPropagation();
        self.applyCompletion(mention);
      });
      item.addEventListener('mouseenter', function(e) {
        e.preventDefault();
        e.stopPropagation();
        self.selectMention(mention);
      });
      return item;
    });
  }

  applyCompletion(mention, state, dispatch) {
    let from = this.spec.from;
    let to = this.spec.to;
    let currentState = state || this.view.state;
    let transaction = function(state) {
      return state.tr.delete(from, to)
        .insert(from, state.schema.node('mention', mention)).insertText(' ');
    };
    if (dispatch) {
      dispatch(transaction(currentState));
    } else {
      this.view.dispatch(transaction(currentState));
    }
    this.destroy();
  }
  selectMention(mention) {
    let oldSelectedMentionIdx = this.selectedMentionIdx;

    if (mention === 'up') {
      if (this.selectedMentionIdx - 1 < 0) {
        this.selectedMentionIdx = this.spec.mentions.length - 1;
      } else {
        this.selectedMentionIdx -= 1;
      }
    } else if (mention === 'down') {
      if (this.selectedMentionIdx + 1 >= this.spec.mentions.length) {
        this.selectedMentionIdx = 0;
      } else {
        this.selectedMentionIdx += 1;
      }
    } else if (mention && mention.username) {
      this.selectedMentionIdx = this.spec.mentions.findIndex(
        m => m.username === mention.username);
    } else {
      this.selectedMentionIdx = 0;
    }

    // update ui
    if (oldSelectedMentionIdx >= 0 ) {
      this.items[oldSelectedMentionIdx].classList.remove('selected');
    }
    if (this.selectedMentionIdx >= 0) {
      this.items[this.selectedMentionIdx].classList.add('selected');
    }
  }
}
