
import elt from '../../../../../node_modules/crel';
import {getIcon} from "./icons";

const prefix = 'ProseMirror-menu';

function translate(view, text) {
  return view._props.translate ? view._props.translate(text) : text;
}

export function renderDropdownItems(items, view) {
  let rendered = [], updates = [];
  for (let i = 0; i < items.length; i++) {
    let {dom, update} = items[i].render(view);
    rendered.push(elt("div", {class: prefix + "-dropdown-item"}, dom));
    updates.push(update);
  }
  return {dom: elt('div', null, rendered), update: combineUpdates(updates, rendered)};
}

function combineUpdates(updates, nodes) {
  return state => {
    let something = false;
    for (let i = 0; i < updates.length; i++) {
      let up = updates[i](state);
      nodes[i].style.display = up ? "" : "none";
      if (up) something = true;
    }
    return something;
  };
}

// Work around classList.toggle being broken in IE11
function setClass(dom, cls, on) {
  if (on) { dom.classList.add(cls); }
  else { dom.classList.remove(cls); }
}


// Largely inspired by MenuItem (see prosemirror-menu) class.
// Few differences :
// - in spec, you can pass a render function instead of the icon or label
// - when user click MenuItem :
//    - if spec.run returns true, nothing more thant the run() happens
//    - if spec.run returns false, it opens a Drop instance
//    - if Drop instance is opened, Drop instance is closed
// - spec.content can be a function
// - Drop instance is closed if :
//    - .on.interaction.dispatch()
//    - editor is blurred
//    - editor is destroyed
//   Thus Drop content shouldn't trigger a drop close directly, but trigger an interaction.dispatch
export class MenuDropItem {
  constructor(spec) {
    this.spec = spec;
  }

  render(view) {
    let spec = this.spec;
    let dom;
    if (spec.render) {
      dom = elt("div", null, spec.render(view));
    } else if (spec.icon) {
      dom = getIcon(spec.icon);
    } else if (spec.label) {
      dom = elt("div", null, translate(view, spec.label));
    } else {
      throw new RangeError("MenuDropItem without render, icon, or label property");
    }

    if (spec.caret) dom.classList.add(prefix + '-dropdown');

    if (spec.title) {
      const title = (typeof spec.title === "function" ? spec.title(view.state) : spec.title);
      dom.setAttribute("title", translate(view, title));
    }
    if (spec.class) dom.classList.add(spec.class);
    if (spec.css) dom.style.cssText += spec.css;

    let close;
    let drop;
    dom.addEventListener(spec.execEvent || "mousedown", e => {
      e.preventDefault(); e.stopPropagation();
      if (close && close()) { close = null; return; }
      if (spec.run && spec.run(view.state, view.dispatch, view, e)) { return; }
      let tmp = this.open(view, dom);
      drop = tmp.drop;
      close = tmp.close;

      drop.on('close', function() {
        if (close && close()) { close = null; }
      });
    });

    function update(state) {
      if (spec.render) {
        while (dom.firstChild) {
          dom.removeChild(dom.firstChild);
        }
        dom.appendChild(spec.render(view));
      }

      if (spec.select) {
        let selected = spec.select(state);
        dom.style.display = selected ? "" : "none";
        if (!selected) {
          if (close && close()) { close = null; }
          return false;
        }
      }
      let enabled = true;
      if (spec.enable) {
        enabled = spec.enable(state) || false;
        setClass(dom, prefix + "-disabled", !enabled);
        if (!enabled) {
          if (close && close()) { close = null; }
        }

      }
      if (spec.active) {
        let active = enabled && spec.active(state) || false;
        setClass(dom, prefix + "-active", active);
      }
      return true;
    }

    return {dom, update};
  }

  open(view, dom) {
    let content;
    if (this.spec.content instanceof Function) {
      content = this.spec.content(view);
    } else content = this.spec.content;

    if (content.update) {
      content.update(view.state);
    }

    let drop = new Drop({
      target: dom,
      content: content.dom,
      position: 'bottom left',
      classes: 'drop-theme-prosemirror',
      constrainToScrollParent: false,
      openOn: null,
      remove: true
    });

    function stopEvent(e) {
      e.preventDefault();
      e.stopPropagation();
    }

    drop.on('open', function() {
      drop.drop && drop.drop.addEventListener('mousedown', stopEvent);
    });

    drop.on('close', function() {
      drop.drop && drop.drop.removeEventListener('mousedown', stopEvent);
    });

    let hadFocus = view.hasFocus();
    function close() {
      view.props.on.interaction.remove(close);
      view.props.on.blur.remove(close);
      view.props.on.editorDestroy.remove(close);
      drop.close();
      if (hadFocus) setTimeout(() => view.focus(), 50);
      return true;
    }
    view.props.on.interaction.dispatch();
    drop.open();
    this.spec.afterOpened && this.spec.afterOpened(view, drop);
    view.props.on.interaction.add(close);
    view.props.on.blur.add(close);
    view.props.on.editorDestroy.add(close);
    return {drop, close};
  }
}
