
function SelectorError() {}

function Selector(pages) {
  this.reset(pages);
}

Selector.prototype.reset = function(pages) {
  this.total = 0;
  this.position = null;
  this.previousPosition = null;
  this.selected = null;
  this.selectedPage = null;
  this.onListeners = [];
  this.offListeners = [];
  this.resetPages(pages || []);
  this.canSelectPages = false;
  this.cycleThroughPagesWhenSelected = false;
  this.blockItemSelectionWhilePageSelected = true;
  return this;
};

Selector.prototype.hasItems = function() {
  return this.total;
};

Selector.prototype.resetPages = function(pages) {
  this.pages = [];
  this.total = 0;
  if (pages) {
    pages.forEach(function(page) {
      this.addPage(page.items, page);
    }, this);
  }
  return this;
};

Selector.prototype._computePageBounds = function() {
  var cumul = 0;
  for (var i = 0; i < this.pages.length; i++) {
    angular.extend(this.pages[i], {
      index: i,
      start: cumul,
      end: cumul + this.pages[i].items.length
    });
    cumul += this.pages[i].items.length;
  }
};

Selector.prototype.addPage = function(items, data) {
  var start = 0;
  if (this.pages.length > 0) {
    start = this.pages[this.pages.length - 1].end;
  }
  this.pages.push(angular.extend({
    index: this.pages.length,
    items: items,
    start: start,
    end: start + items.length
  }, data || {}));
  this.total += items.length;
  return this;
};

Selector.prototype.matchPage = function(a, b) {
  return a && b && (a === b || StoreQuery.matchObject(a, b));
};

Selector.prototype.findPage = function(props) {
  if (typeof(props) === 'number') {
    return this.pages[props];
  }
  var match = false;
  for (var i = 0; i < this.pages.length; i++) {
    if (this.matchPage(this.pages[i], props)) {
      return this.pages[i];
    }
  }
};

Selector.prototype.replacePageItems = function(page, items = []) {
  page = this.findPage(page);
  if (page) {
    this.total = this.total - page.items.length + items.length;
    page.items = items;
    this._computePageBounds();
    if (this.selected) {
      this._select(this.selected);
    }
  }
};

Selector.prototype.removePage = function(page) {
  page = this.findPage(page);
  if (page) {
    this.pages.splice(page.index, 1);
    this.total -= page.items.length;
    this._computePageBounds();
    if (this.selected) {
      this._select(this.selected);
    }
  }
};

Selector.prototype.getPageFromPosition = function(position) {
  for (var i = 0; i < this.pages.length; i++) {
    if (position >= this.pages[i].start && position < this.pages[i].end) {
      return this.pages[i];
    }
  }
  return null;
};

Selector.prototype.getCurrentPage = function() {
  if (this.selectedPage) {
    return this.selectedPage;
  }
  return this.getPageFromPosition(this.position);
};

Selector.prototype._move = function(delta) {
  var pos = this.position;
  if (pos === null) {
    pos = this.previousPosition || 0;
  }
  if (pos === 0 && delta < 0) {
    pos = this.total - 1;
  } else if (pos === this.total - 1 && delta > 0) {
    pos = 0;
  } else {
    pos += delta;
    if (pos < 0) {
      pos = 0;
    } else if (pos >= this.total) {
      pos = this.total - 1;
    }
  }
  return this.selectByPosition(pos);
};

Selector.prototype.prev = function(delta) {
  if (!this.selectedPage && this.position === null && this.previousPosition === null) {
    return this.selectByPosition(this.total - 1);
  }
  return this._move(-(delta || 1));
};

Selector.prototype.next = function(delta) {
  if (!this.selectedPage && this.position === null && this.previousPosition === null) {
    return this.selectByPosition(0);
  }
  return this._move(delta || 1);
};

Selector.prototype._movePage = function(delta) {
  var pos = this.position;
  if (pos === null) {
    pos = this.previousPosition || 0;
  }
  var page = this.getCurrentPage();
  var delta1 = delta/Math.abs(delta);

  if (!this.selectedPage && (this.pages.length === 1 || !page)) {
    return this._move(delta1);
  }

  var pageIndex = page.index + delta;
  var pagesCount = this.pages.length;
  if (pageIndex < 0) {
    pageIndex = pagesCount - (pagesCount + (- pageIndex)) % pagesCount;
  } else if (pageIndex >= pagesCount) {
    pageIndex = (pageIndex + 1) % (pagesCount) - 1;
  }

  if (this.cycleThroughPagesWhenSelected && this.selectedPage) {
    return this.selectPage(this.pages[pageIndex], this.selectedPageSection);
  }

  var i = pageIndex;
  while (this.pages[i].items.length === 0) {
    i += delta1;
    if (i >= this.pages.length) {
      i = 0;
    }
    if (i < 0) {
      i = this.pages.length - 1;
    }
    if (i === pageIndex) {
      break;
    }
  }
  var newPage = this.pages[i];

  var deltaStartPage = Math.max(pos - page.start, 0);
  pos = newPage.start + deltaStartPage;
  if (pos >= newPage.end) {
    pos = newPage.end - 1;
  }
  return this.selectByPosition(pos);
};

Selector.prototype.prevPage = function(delta) {
  if (!this.selectedPage && this.position === null && this.previousPosition === null) {
    return this.selectByPosition(0);
  }
  return this._movePage(-(delta || 1));
};

Selector.prototype.nextPage = function(delta) {
  if (!this.selectedPage && this.position === null && this.previousPosition === null) {
    return this.selectByPosition(0);
  }
  return this._movePage(delta || 1);
};

Selector.prototype.selectPage = function(page, section, takeOverPage) {
  page = this.findPage(page);
  takeOverPage = typeof(takeOverPage) === 'undefined' || takeOverPage;
  if (this.canSelectPages) {
    if (this.selectedPage === page) {
      return;
    }
    this.unselect();
    this.selectedPage = page;
    this.selectedPageSection = section;
    this._dispatch(this.onListeners, null, page);
  } else {
    this.selectFirstOfPage(page.index, takeOverPage);
  }
};

Selector.prototype.unselectPage = function() {
  if (!this.selectedPage) return;
  this.unselect();
  if (this.previousPosition !== null) {
    this.selectByPosition(this.previousPosition);
  }
};

Selector.prototype.selectFirstOfPage = function(page, takeOverPage) {
  page = this.findPage(page);
  if (!page) {
    throw new SelectorError('Unknown page');
  }

  if (
    page.end === page.start &&
    !takeOverPage
  ) {
    return;
  }

  return this.selectByPosition(page.start, takeOverPage);
};

Selector.prototype.selectLastOfPage = function(page, takeOverPage) {
  page = this.findPage(page);
  if (!page) {
    throw new SelectorError('Unknown page');
  }

  if (
    page.end === page.start &&
    !takeOverPage
  ) {
    return;
  }

  return this.selectByPosition(Math.max(page.end - 1, page.start), takeOverPage);
};

Selector.prototype.select = function(item, takeOverPage, selectIfNotFound) {
  if ((!takeOverPage && this.selectedPage && this.blockItemSelectionWhilePageSelected) || this.selected === item) {
    return;
  }
  return this._select(item, takeOverPage, selectIfNotFound);
};

Selector.prototype._select = function(item, takeOverPage, selectIfNotFound) {
  var pos = 0;
  for (var i = 0; i < this.pages.length; i++) {
    for (var j = 0; j < this.pages[i].items.length; j++) {
      if (this.pages[i].items[j] === item) {
        this.unselect();
        return this.selectByPosition(pos, takeOverPage);
      }
      pos++;
    }
  }
  if (selectIfNotFound) {
    this.unselect();
    this.selected = item;
    this._dispatch(this.onListeners, this.selected);
    return item;
  }
};

Selector.prototype.selectByPosition = function(pos, takeOverPage) {
  if (this.position === pos) {
    return;
  }
  if (this.selectedPage) {
    if (this.blockItemSelectionWhilePageSelected && !takeOverPage) {
      return;
    }
    this._dispatch(this.offListeners, null, this.selectedPage);
    this.selectedPage = null;
  } else if (this.selected) {
    this.unselect();
  }
  this.position = pos;
  var page = this.getPageFromPosition(pos);
  this.selected = page.items[pos - page.start];
  this._dispatch(this.onListeners, this.selected);
  return this.selected;
};

Selector.prototype.unselect = function() {
  if (this.selected) {
    // if (this.position !== null) {
    //   this.previousPosition = this.position;
    // }
    this.position = null;
    this._dispatch(this.offListeners, this.selected);
  } else if (this.selectedPage) {
    this._dispatch(this.offListeners, null, this.selectedPage);
  }
  this.selected = null;
  this.selectedPage = null;
  this.selectedPageSection = null;
};

Selector.prototype._dispatch = function(listeners, item, page) {
  listeners.forEach(function(cb) {
    cb.call(this, item, page);
  }, this);
};

Selector.prototype._addListener = function(listeners, item, page, callback) {
  var cb;
  if (typeof(item) === 'function') {
    cb = item;
  } else {
    if (typeof(page) === 'function') {
      callback = page;
      page = null;
    }
    var self = this;
    cb = function(obj, p) {
      if (item === obj && (!page || self.matchPage(p, page))) {
        callback(obj, p);
      }
    };
  }
  listeners.push(cb);
  return function() {
    listeners.splice(listeners.indexOf(cb), 1);
  };
};

Selector.prototype.on = function(item, page, callback) {
  return this._addListener(this.onListeners, item, page, callback);
};

Selector.prototype.$on = function(scope, item, page, callback) {
  scope.$on('$destroy', this.on(item, page, function(item) {
    callback(item);
  }));
};

Selector.prototype.off = function(item, page, callback) {
  return this._addListener(this.offListeners, item, page, callback);
};

Selector.prototype.$off = function(scope, item, page, callback) {
  scope.$on('$destroy', this.off(item, page, function(item) {
    callback(item);
  }));
};
